//   RW Sumomo
// ==============
// Demonstration of a Chibitsu-grade robowaifu
// -Part of the RW Foundations suite of libraries

// Filename: ./include/chibitsu/sumomo.hpp
// -Declarations of the baseline RW Sumomo project subsystem

#pragma once

#include "incls_all.hpp"

namespace rw::chibitsu {

//------------------------------------------------------------------------------
/**
 */
class Sumomo_b : public Chibitsu {
 public:
  Sumomo_b();                                    // default ctor
  explicit Sumomo_b(std::string const& handle);  // tag-parm'd ctor
};

//------------------------------------------------------------------------------
/**
 */
class Sumomo : public Sumomo_b {
 public:
  Sumomo();                                    // default ctor
  explicit Sumomo(std::string const& handle);  // tag-parm'd ctor

  auto do_preflight() -> bool override;

  inline auto confetti() const -> Confetti& { return confetti_; }
  inline auto dustpan() const -> Dustpan& { return dustpan_; }
  inline auto flags() const -> Flags& { return flags_; }
  inline auto pompoms() const -> Pompoms& { return pompoms_; }
  inline auto tambourine() const -> Tambourine& { return tambourine_; }
  inline auto whisk() const -> Whisk& { return whisk_; }
  inline auto whistle() const -> Whistle& { return whistle_; }

 private:
  unique_ptr<Confetti_b> _confetti_{make_unique<Confetti_b>("sumomo_confetti")};
  unique_ptr<Dustpan_b>  _dustpan_{make_unique<Dustpan_b>("sumomo_dustpan")};
  unique_ptr<Flags_b>    _flags_{make_unique<Flags_b>("sumomo_flags")};
  unique_ptr<Pompoms_b>  _pompoms_{make_unique<Pompoms_b>("sumomo_pompoms")};
  unique_ptr<Tambourine_b> _tambourine_{
      make_unique<Tambourine_b>("sumomo_tambourine")};
  unique_ptr<Whisk_b>   _whisk_{make_unique<Whisk_b>("sumomo_whisk")};
  unique_ptr<Whistle_b> _whistle_{make_unique<Whistle_b>("sumomo_whistle")};

  Confetti&   confetti_{reference_to<Confetti>(_confetti_.get())};
  Dustpan&    dustpan_{reference_to<Dustpan>(_dustpan_.get())};
  Flags&      flags_{reference_to<Flags>(_flags_.get())};
  Pompoms&    pompoms_{reference_to<Pompoms>(_pompoms_.get())};
  Tambourine& tambourine_{reference_to<Tambourine>(_tambourine_.get())};
  Whisk&      whisk_{reference_to<Whisk>(_whisk_.get())};
  Whistle&    whistle_{reference_to<Whistle>(_whistle_.get())};
};

}  // namespace rw::chibitsu

//    --=^=--

// "EMERGENCY, EMERGENCY! THIS IS A CRISIS SITUATION!!"

// rw_chibitsu_lib.v220212
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
