// RW Bumpmaster Project
// =====================
// Imageboard downloader + archiver demo project;  SAVE.EVERYTHING.ANON. :^)

// Filename: ./main_bumpmaster.cpp
// -The program's entry point

#include "incls_all.hpp"

void diag_bumpmaster(Bumpmaster const& bump);

void prt_ten_five(Bumpmaster&        bump,
                  Json const&        catalog,
                  std::string const& targ_field,
                  std::string const& cat_uri);

void prt_targ_fields(Json const& json, std::string const& targ_field);

//------------------------------------------------------------------------------
/**
 */
int main() try {
  puts("\n SAVE.EVERYTHING.ANON. :^)");

  Bumpmaster bump{"bumpmaster"};
  diag_bumpmaster(bump);

  puts("\n\n     --=^=--\n\n");

  // TEMP: TEST: retrieve some board's Page 1 threads, and display each of these
  // thread's five newest post's message text (kind of like a board index :^)

  vector<string> const catalog_uris{

      "https://alogs.space/robowaifu/catalog.json",
      "https://prolikewoah.com/geimu/catalog.json",
      "https://late.city/late/catalog.json",
      "https://anon.cafe/christian/catalog.json",

  };

  for (auto const& cat_uri : catalog_uris) {  // cycles through all catalog uris

    auto cat_stream = bump.dolldrop().pull(cat_uri);
    if (! is_empty(cat_stream)) {  //

      auto const catalog = fstream_2_json(cat_stream);
      if (! catalog.empty()) {  //

        puts("\n\n     --=^=--\n\n");

        cout << " thread count :  " << catalog.size() << "\n\n";

        // Lynxchan's catalog's thread's JSON field names:
        /*
        "autoSage"
        "cyclic"
        "fileCount"
        "lastBump"
        "locked"
        "markdown"
        "message"
        "mime"
        "page"
        "pinned"
        "postCount"
        "subject"
        "threadId"
        "thumb"
        */
        string const field_nm{"subject"};
        prt_ten_five(bump, catalog, field_nm, cat_uri);

        puts("\n\n     --=^=--\n\n");

      } else
        cerr << "\n\n  empty JSON data\n\n";

    } else
      cerr << "\n\n  empty file stream\n\n";
  }

  // if (use_stamps)
  //   analyze_callgraph();

  // puts("\n\n     --=^=--\n\n");

  puts(" ...leaving main()\n");

} catch (runtime_error const& e) {
  cerr << "\n  runtime_error '" << e.what() << "' caught in main()\n\n";

} catch (exception const& e) {
  cerr << "\n  exception '" << e.what() << "' caught in main()\n\n";

} catch (...) {
  cerr << "\n  unknown error caught in main()\n\n";
}

//------------------------------------------------------------------------------
/**
 * @param bump
 */
void diag_bumpmaster(Bumpmaster const& bump) {  //

  prt_tag(bump.t(), "bump");
  prt_tag(bump.board().t(), "bump.board()");
  prt_tag(bump.catalog().t(), "bump.catalog()");
  prt_tag(bump.page().t(), "bump.page()");
  prt_tag(bump.post().t(), "bump.post()");
  prt_tag(bump.site().t(), "bump.site()");
  prt_tag(bump.thread().t(), "bump.thread()");
  prt_tag(bump.waifusearch().t(), "bump.waifusearch()");
  //
  prt_tag(bump.dolldrop().t(), "bump.dolldrop()");
}

//------------------------------------------------------------------------------
/** Extract site name, board name, from IB catalog URI @a cat_uri
 * @param cat_uri
 * @return
 */
std::tuple<std::string, std::string> parse_cat_uri(std::string const& cat_uri) {
  string site{""}, board{""};

  regex const ex{"(http|https)://([^/ ]*)/([^ #?]*)/([^ #?]*)"};
  std::cmatch match;

  if (regex_match(cat_uri.c_str(), match, ex)) {
    site  = string{match[2].first, match[2].second};
    board = string{match[3].first, match[3].second};
  }

  return make_tuple(site, board);
}

//------------------------------------------------------------------------------
/** Assemble an IB thread JSON URI from key parts
 * @param host
 * @param path
 * @param thrd_id
 * @return the assembled URI
 * @note: example resulting form: https://alogs.space/robowaifu/res/14409.json
 */
std::string build_thread_uri(std::string const& host,
                             std::string const& path,
                             unsigned const     thrd_id) {
  string const scheme{"https://"};
  string const res{"res"};
  string const type{"json"};

  return {scheme + host + '/' + path + '/' + res + '/' + to_string(thrd_id) +
          '.' + type};
}

//------------------------------------------------------------------------------
/** Pull thread's JSON dolldrop
 * @param thread_uri
 * @param bump
 * @return @c Json object constructed from the dolldrop
 */
Json pull_thread_json(Bumpmaster& bump, std::string const& thread_uri) {  //

  auto thrd_stream = bump.dolldrop().pull(thread_uri);
  if (! is_empty(thrd_stream)) {  //

    auto const thrd_json = fstream_2_json(thrd_stream);
    if (! thrd_json.empty()) {  //

      return thrd_json;

    } else
      cerr << "\n\n  empty JSON data - pull_thread_json()\n\n";

  } else
    cerr << "\n\n  empty file stream - pull_thread_json()\n\n";

  return Json{};
}

//------------------------------------------------------------------------------
/** Print thread's newest posts out to console
 * @param thrd_json
 */
void prt_post_msgs(Json const& thrd_json) {  //

  int const limit{5};  // just the 5 newest posts

  int    postId{0};
  string creation{""}, message{""};

  for (auto const& [field_nm, field_dat] : thrd_json.items()) {  //

    if (field_nm == "posts") {  //
      auto const sz = static_cast<int>(field_dat.size());

      for (auto const& [post_idx, post_dat] : field_dat.items()) {  //

        // display only the newest posts from the thread
        if (stoi(post_idx) >= (sz - limit)) {  //

          for (auto const& [fld_nm, fld_dat] : post_dat.items()) {  //

            // collect up post info for re-ordered printing next

            if (fld_nm == "creation")
              creation = fld_dat;

            if (fld_nm == "message")
              message = fld_dat;

            if (fld_nm == "postId")
              postId = fld_dat;
          }

          cout << "\n  ------------------------------------\n"
               << setw(8) << setfill(' ') << postId << "  :  " << creation
               << "\n\t' " << message << " '\n";
        }
      }
    }
  }

  puts("\n");
}

//------------------------------------------------------------------------------
/** Print the board's Page 1 thread's 5 latest post messages out to console
 * @param bump
 * @param catalog
 * @param targ_field
 * @param cat_uri
 * @note: this will pull dolldrops for all the board's Page 1 threads as well
 */
void prt_ten_five(Bumpmaster&        bump,
                  Json const&        catalog,
                  std::string const& targ_field,
                  std::string const& cat_uri) {  //

  unsigned const limit{10};  // just the Page 1 threads

  // split out the site name, board name from the URI (for building thread URIs)
  auto const& [site, board] = parse_cat_uri(cat_uri);

  // cycle through the limited-count of the board's threads
  for (unsigned thrd_idx{0}; thrd_idx < limit; ++thrd_idx) {
    auto const& curr_thrd = catalog[thrd_idx];

    // cycle through the thread's fields
    for (auto const& [field_nm, field_dat] : curr_thrd.items()) {  //

      // print out both the thread's 'catalog bump-index', and it's thread ID

      if (field_nm == targ_field)
        cout << "\n =======================================\n\n " << setw(2)
             << setfill(' ') << (thrd_idx + 1) << ' ' << field_nm << "  :  "
             << field_dat << "\n";

      if (field_nm == "threadId") {
        cout << ' ' << setw(10) << setfill(' ') << field_nm << "  :  "
             << field_dat << "\n\n";

        // build the URI and pull the thread's JSON dolldrop
        auto const& thread_uri = build_thread_uri(site, board, field_dat);
        auto const& thrd_json  = pull_thread_json(bump, thread_uri);

        // print the thread's post messages
        prt_post_msgs(thrd_json);
      }
    }
  }
}

//------------------------------------------------------------------------------
/** For each thread in @a catalog, print out specified target field's value data
 * @param catalog the @c Json to search through (an IB catalog)
 * @param targ_field the @c std::string to match against
 * @note: this is mostly just for Lynxchan or JSchan boards, but can be changed
 */
void prt_targ_fields(Json const& catalog, std::string const& targ_field) {
  // NOTE: these C++17 structured bindings represent JSON {key:value} pairs

  // NOTE: a third, outer loop would start here (eg, for vichan etc., boards)
  // ...

  // cycle through all the board catalog's threads (loop 1)
  for (auto const& [implct_idx, thread] : catalog.items()) {  //

    // for each thread, cycle through all of it's fields (loop 2)
    for (auto const& [field_nm, field_dat] : thread.items()) {  //

      // finally, print out data for the thread's matching target field
      if (field_nm == targ_field)
        cout << ' ' << setw(3) << setfill(' ') << (stoi(implct_idx) + 1) << ' '
             << field_nm << "  :  " << field_dat << "\n";
    }
  }
}

//    --=^=--

// "Everyone's talking about IT & the Internet, and how advanced it's getting."

// rw_bumpmaster_demo.v220212
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
