//  RW Foundations
// ================
// ______
// -The RW Foundational Libraries

// Filename: ./src/rw/rw.cpp
// -Definitions of the baseline RW library

#include "incls_all.hpp"

namespace rw {

//------------------------------------------------------------------------------
//    RW
//------------------------------------------------------------------------------
RW::RW() {}

//------------------------------------------------------------------------------
RW::RW(std::string const& handle, Type const type)
    : hip_{handle, rw_id++, this, type},  //
      do_stamps_{use_stamps}              //
{
  stamp("RW()");

  RW_objs.push_back(this);  // track this RW object in the 'actives' container
}

//------------------------------------------------------------------------------
RW::~RW() {
  // locate the object's this pointer in the 'actives' container & null it out
  if (auto it = find(begin(RW_objs), end(RW_objs), this); it != end(RW_objs))
    *it = nullptr;
}

//------------------------------------------------------------------------------

// TODO: inline all these RW:: funcs, except do_stamps(bool)
// -when we do this, also consider things w/ tightening up all returns to const
// ...

//------------------------------------------------------------------------------
auto RW::hdl() const -> std::string {
  return hip_.hdl;
}

//------------------------------------------------------------------------------
auto RW::id() const -> unsigned int {
  return hip_.id;
}

//------------------------------------------------------------------------------
auto RW::addr() const -> RW* {
  return hip_.ptr;
}

//------------------------------------------------------------------------------
auto RW::typ() const -> Type {
  return hip_.typ;
}

//------------------------------------------------------------------------------
auto RW::t() const -> Hip const {
  return hip_;
}

//------------------------------------------------------------------------------
void RW::stamp(const Hip& tag, std::string const& func_nm) {  //

  if (do_stamps_)
    stamps_.emplace(Callstamp{tag, func_nm, Call_clk::now()});
}
// self-use wrapper
void RW::stamp(std::string const& func_nm) {
  stamp(hip_, func_nm);
}

//------------------------------------------------------------------------------
void RW::do_stamps(Hip const& tag, const bool flag) {
  stamp(tag, "do_stamps()");

  // if (tag == hip_) {  // self-use only  // DESIGN: rework to collection of
  do_stamps_ = flag;

  // } else
  //   error("this setting is only for self-use", hip_, "RW::do_stamps()");
}

//------------------------------------------------------------------------------
unsigned long int RW::rw_id{0};  // private static member

//------------------------------------------------------------------------------
//    RW_slave
//------------------------------------------------------------------------------
RW_slave::RW_slave(std::vector<Type> const& masters, std::string const& handle)
    : RW_slave{masters, handle, Type::Foo} {}

//------------------------------------------------------------------------------
RW_slave::RW_slave(std::vector<Type> const& masters,
                   std::string const&       handle,
                   Type const               type)
    : RW{handle, type} {
  stamp("RW_slave()");

  master_types(masters);
}

//------------------------------------------------------------------------------
auto RW_slave::bind_master(Hip const& tag) -> bool {  //
  stamp(tag, "bind_master()");

  /** Determines if this caller's tag @c Type is included anywhere within the
   * list of acceptable @a Master @c Type 's for this *slave object* class
   * @note: list is assigned to in the @c master_types() call, made during this
   * *slave object* class' constructor
   */
  auto const suitable_master =
      any_of(cbegin(master_types_), cend(master_types_),
             [&](auto const type) { return type == tag.typ; });

  if (suitable_master) {           // This would be a good master for slave obj,
    if (master_.ptr == nullptr) {  // this slave object not bound to a master,
      master_ = tag;               // so bind to this caller;
      return true;                 // slave object accepts 'tag' as Master.
                                   //
    } else                         // pointer set, slave already owned by Master
      rw_error("I already belong to Master", t(), "RW_slave::bind_master()");

  } else  // lol no, go away stranger  :^)
    rw_error("I can't accept you as Master", t(), "RW_slave::bind_master()");

  ///
  return false;  // shouldn't reach here
}

//------------------------------------------------------------------------------
void RW_slave::master_types(std::vector<Type> const& masters) {
  stamp("master_types()");

  master_types_ = masters;
}

}  // namespace rw

//    --=^=--

// "So is it true Sempai? Are robowaifus really better than any human woman?"

// rw_foundations_lib.v220212
// Copyright (2022)
// License MIT(Expat) https://opensource.org/licenses/MIT
